@@ -29,6 +29,7 @@ use constant {
   DOCUMENT   => "\x03",
   ARRAY      => "\x04",
   BINARY     => "\x05",
+  UNDEFINED  => "\x06",
   OBJECT_ID  => "\x07",
   BOOL       => "\x08",
   DATETIME   => "\x09",
@@ -39,8 +40,8 @@ use constant {
   INT32      => "\x10",
   TIMESTAMP  => "\x11",
   INT64      => "\x12",
-  MIN_KEY    => "\x7f",
-  MAX_KEY    => "\xff"
+  MIN_KEY    => "\xff",
+  MAX_KEY    => "\x7f"
 };
 
 # Binary subtypes
@@ -211,9 +212,13 @@ sub _decode_value {
     if $type eq DATETIME;
 
   # Regex
-  return eval join '/', 'qr', _decode_cstring($bsonref),
-    _decode_cstring($bsonref)
-    if $type eq REGEX;
+  if ($type eq REGEX) {
+    my ($p, $m) = (_decode_cstring($bsonref), _decode_cstring($bsonref));
+    croak "invalid regex modifier(s) in 'qr/$p/$m'"
+      if length($m) and $m !~ /^[msixpadlun]+\z/;
+    # escape $pat to avoid code injection
+    return eval "qr/\$p/$m";
+  }
 
   # Binary (with subtypes)
   return _decode_binary($bsonref) if $type eq BINARY;
@@ -234,6 +239,10 @@ sub _decode_value {
     reverse map({ unpack 'l<', substr($$_, 0, 4, '') } $bsonref, $bsonref))
     if $type eq TIMESTAMP;
 
+  # Undefined - a deprecated type which should not exist anymore
+  # but apparently still does: https://github.com/oliwer/mango/issues/1
+  return undef if $type eq UNDEFINED;
+
   # Unknown
   croak 'Unknown BSON type';
 }
